'use strict';

require('rootpath')();
const Errors = require('errors');
const Serializer = require('@dp/mapi-serializer');

/**
 * 判断是否有后缀
 */
function hasSuffix(context){
    return (context && context.params && context.params.suffix);
}

/**
 * 成功执行的返回逻辑
 * @param  string suffix url的后缀（例如：json,jsonp,bin等）
 * @param  object data   业务逻辑执行结果
 * @param  koa    app    KOA上下文
 */
function *responseSuccess(suffix, data, app) {
    if (suffix === 'json') {
        app.body = yield Promise.resolve(JSON.stringify(createResponseJson(200, data)));
    } else if (suffix === 'jsonp') {
        let callbackValue = app.query['callback'];
        app.body = yield Promise.resolve(callbackValue + '(' + JSON.stringify(createResponseJson(200, data)) + ')');
    } else if (suffix === 'bin') {
        let res = app.response && app.response.res;
        res.setHeader('Content-Type', 'application/binary; charset=utf-8');
        res.statusCode = 200;

        let bufArr = yield Serializer.encode(data);
        res.setHeader('Content-Length', bufArr.length);
        res.write(bufArr);
        res.end();
    }
}

/**
 * 如果是提供给MAPI的数据对象，需要进行转换
 */
function createResponseJson(code, data) {
    return {
        code: code,
        msg: !Serializer.isMAPIType(data) ? data : data.toJson()
    };
}

/**
 * 异常执行的返回逻辑
 * @param  string suffix url的后缀（例如：json,jsonp,bin等）
 * @param  int    errorCode   错误码
 * @param  string msg   错误描述信息
 * @param  koa    app   KOA上下文
 */
function *responseError(suffix, errorCode, msg, app) {
    if (suffix === 'json') {
        app.body = yield Promise.resolve(JSON.stringify(createResponseJson(errorCode, msg)));
    } else if (suffix === 'jsonp') {
        let callbackValue = app.query['callback'];
        app.body = yield Promise.resolve(callbackValue + '(' + JSON.stringify(createResponseJson(errorCode, msg)) + ')');
    } else if (suffix === 'bin') {
        let res = app.response && app.response.res;
        res.setHeader('Content-Type', 'application/binary; charset=utf-8');
        res.statusCode = 200;

        let data = new Serializer.SimpleMsg();

        let bufArr = yield Serializer.encode(data);
        res.setHeader('Content-Length', bufArr.length);
        res.write(bufArr);
        res.end();
    }
}

module.exports = function *(next) {
    this.skipCortex = true;
    // api执行错误，直接抓取异常
    try {
        yield next;
        // 没有后缀，直接返回
        if (!hasSuffix(this)){
            return;
        }

        // 根据后缀决定执行逻辑
        let suffix = this.params && this.params.suffix;
        yield responseSuccess(suffix, this.body, this);
    } catch (e) {
        let suffix = this.params && this.params.suffix;
        if (e instanceof Errors.ApplicationError) {
            yield responseError(suffix, e.errorCode, e.message, this);
        } else {
            console.log(e.stack);
            yield responseError(suffix, 500, 'error happened', this);
        }
    }
};
