'use strict';

require('rootpath')();

const _ = require('lodash');
const render = require('./render').render;
const async = require('@dp/vc-async');
const register = require('./register');
const cortex = require('./cortex');

/**
 * 首页渲染接口
 * @param koa
 * @param params
 * @param widgetNames
 * @param options
 */
function *execute(koa, params, widgetNames, options) {

    preprocess(koa);

    let widgets = _.map(widgetNames, function(name){
        return register.getByName(name);
    });

    let widgetResults = yield executeWisgets(params, widgets);

    let data = yield buildCortexData(koa, widgets, widgetResults, options);

    let body = yield render(options.htmlPath, data);

    koa.body = body;
}

/**
 * 预处理
 * @param koa
 */
function preprocess(koa) {
    koa.skipCortex = true;
}

/**
 * 渲染widgets
 * @param params
 * @param widgets
 * @returns {Array}
 */
function *executeWisgets(params, widgets) {
    let tasks = buildTasks(params, widgets);
    let results = yield async.parallel(tasks);
    return _.values(results);
}

/**
 * 创建paralle task
 * @param params
 * @param widgets
 * @returns {{}}
 */
function buildTasks(params, widgets) {
    let result = {};
    _.each(widgets, (widget) => {
        result[widget.name] = {
            *gen() {
                return yield widget.execute(params);
            },
            fallback: null
        };
    });
    return result;
}

/**
 * 根据widgets以及首页option配置的参数获取cortex引入的内容
 * @param koa
 * @param widgets
 * @param widgetsResults
 * @param options
 * @returns {{facadesCssPlaceHolder: *, content: string, jsFramework: *, facadesPlaceHolder: *}}
 */
function *buildCortexData(koa, widgets, widgetsResults, options) {
    let cortexResult = yield cortex.loadCortexResults(koa, widgets, options);
    return {
        facadesCssPlaceHolder: cortexResult.facadesCssPlaceHolder,
        content: widgetsResults.join(''),
        jsFramework: cortexResult.jsFramework,
        facadesPlaceHolder: cortexResult.facadesPlaceHolder
    };
}

module.exports = {
    execute
};
